<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Observer;

use Magento\Framework\Event\ObserverInterface;
use Amasty\GeoipRedirect\Model\Source\Logic;
use Magento\Store\Model\ScopeInterface;

class Redirect extends \Amasty\GeoipRedirect\Observer\Redirect {

    protected $_helper;

    public function __construct(
    \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress, \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig, \Amasty\GeoipRedirect\Helper\Data $geoipHelper, \Magento\Framework\UrlInterface $urlBuilder, \Magento\Store\Model\StoreManagerInterface $storeManager, \Amasty\Geoip\Model\Geolocation $geolocation, \Magento\Customer\Model\Session $customerSession, \Magento\Store\Api\StoreCookieManagerInterface $storeCookieManager, \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper, \FiloBlu\ExtAmastyGeoipRedirect\Model\Logger\Logger $logger
    ) {
        $this->_helper = $helper;
        $this->_logger = $logger;

        parent::__construct($remoteAddress, $scopeConfig, $geoipHelper, $urlBuilder, $storeManager, $geolocation, $customerSession, $storeCookieManager);
    }

    public function execute(\Magento\Framework\Event\Observer $observer) {

        $controller = $observer->getControllerAction();

        if ($controller->getRequest()->isXmlHttpRequest()) {
            return;
        }

        $currentStoreId = $this->storeManager->getStore()->getId();
        $scopeStores = ScopeInterface::SCOPE_STORES;

        $logEnabled = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log', $scopeStores, $currentStoreId);

        $ipRestriction = $this->scopeConfig->getValue('amgeoipredirect/restriction/ip_restriction');
        $currentIp = $this->remoteAddress->getRemoteAddress();
        $fakeIp = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_ip', $scopeStores, $currentStoreId);
        $fakeCountry = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_country', $scopeStores, $currentStoreId);

        if ($logEnabled) {
            $this->_logger->info(" #### START REQUEST #### ");
        }


        if ($logEnabled) {
            $this->_logger->info("Current Ip: " . $currentIp);

            if ($fakeIp) {
                $this->_logger->info("Fake Ip: " . $fakeIp);
            }

            if ($fakeCountry) {
                $this->_logger->info("Fake Country: " . $fakeCountry);
            }
        }

        if ($fakeIp) {
            $currentIp = $fakeIp;
        }


        if (!empty($ipRestriction)) {
            $ipRestriction = array_map("rtrim", explode(PHP_EOL, $ipRestriction));
            foreach ($ipRestriction as $ip) {
                if ($currentIp && $currentIp == $ip) {
                    if ($logEnabled) {
                        $this->_logger->info("No Redirect For Ip restriction " . $ip);
                    }
                    return;
                }
            }
        }
        $isApi = $controller->getRequest()->getControllerModule() == 'Mage_Api';
        if ($isApi || !$this->geoipHelper->isModuleOutputEnabled('Amasty_Geoip')) {
            if ($logEnabled) {
                $this->_logger->info("No Redirect For No Output enabled Module or API Call");
            }
            return;
        }
        $userAgent = $controller->getRequest()->getHeader('USER_AGENT');
        $userAgentsIgnore = $this->scopeConfig->getValue('amgeoipredirect/restriction/user_agents_ignore');
        if (!empty($userAgentsIgnore)) {
            $userAgentsIgnore = explode(',', $userAgentsIgnore);
            $userAgentsIgnore = array_map("trim", $userAgentsIgnore);
            foreach ($userAgentsIgnore as $agent) {
                if ($userAgent && $agent && stripos($userAgent, $agent) !== false) {
                    if ($logEnabled) {
                        $this->_logger->info("No Redirect For Not Allowed User agent " . $userAgent);
                    }
                    return;
                }
            }
        }

        $this->applyLogic($controller);

        if ($logEnabled) {
            $this->_logger->info("Is redirect Allowed ->" . $this->redirectAllowed);
        }

        if ($this->scopeConfig->getValue('amgeoipredirect/general/enable', $scopeStores, $currentStoreId) && $this->redirectAllowed) {

            $location = $this->geolocation->locate($currentIp);
            $country = $location->getCountry();

            if ($fakeCountry) {
                $country = $fakeCountry;
            }

            if ($logEnabled) {
                $this->_logger->info("Current Country: " . $country);
            }

            if ($country) {
                $this->_helper->setLocalizedCountryCookie($country);
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/restriction/first_visit_redirect')) {
                $getAmYetRedirectStore = $this->_helper->getStoreCookie();
                $getAmYetRedirectCurrency = $this->_helper->getCurrencyCookie();
                $getAmYetRedirectUrl = $this->_helper->getUrlCookie();
            } else {
                $getAmYetRedirectStore = 0;
                $getAmYetRedirectCurrency = 0;
                $getAmYetRedirectUrl = 0;
            }

            if (!$getAmYetRedirectUrl && $this->scopeConfig->getValue('amgeoipredirect/country_url/enable_url')) {
                $urlMapping = unserialize($this->scopeConfig->getValue('amgeoipredirect/country_url/url_mapping', 'default', $currentStoreId));
                $currentUrl = $this->urlBuilder->getCurrentUrl();
                foreach ($urlMapping as $countries => $url) {
                    if (strpos($countries, $country) !== false && $url != $currentUrl) {
                        $this->_helper->setUrlCookie();
                        $controller->getResponse()->setRedirect($url);
                        $controller->getResponse()->sendResponse();
                        exit;
                    }
                }
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/country_store/enable_store')) {
                if ($getAmYetRedirectStore) {
                    if ($getAmYetRedirectStore != $this->storeManager->getStore()->getId()) {

                        if ($logEnabled) {
                            $this->_logger->info("Cookie is set, redirect to ->" . $this->storeManager->getStore($getAmYetRedirectStore)->getBaseUrl());
                        }

                        $this->storeCookieManager->setStoreCookie($this->storeManager->getStore($getAmYetRedirectStore));
                        $controller->getResponse()->setRedirect($this->storeManager->getStore($getAmYetRedirectStore)->getBaseUrl());
                        $controller->getResponse()->sendResponse();
                    }
                } else {


                    if ($country) {
                        $this->_helper->setCountryCookie($country);

                        if ($logEnabled) {
                            $this->_logger->info("Set country cookie->" . $country);
                        }
                    } else {
                        $countryid = $this->scopeConfig->getValue('general/country/default', $scopeStores);
                        $this->_helper->setCountryCookie($countryid);
                        if ($logEnabled) {
                            $this->_logger->info("Set default country cookie->" . $country);
                        }
                    }

                    $allStores = $this->storeManager->getStores();
                    foreach ($allStores as $store) {
                        $currentStoreUrl = str_replace('&amp;', '&', $store->getCurrentUrl(false));
                        $redirectStoreUrl = trim($currentStoreUrl, '/');
                        $countries = $this->scopeConfig->getValue('amgeoipredirect/country_store/affected_countries', $scopeStores, $store->getId());
                        if (!$this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_between_websites')) {
                            $useMultistores = $store->getWebsiteId() == $this->storeManager->getStore()->getWebsiteId();
                        } else {
                            $useMultistores = true;
                        }

                       
                        if ($country && $countries && strpos($countries, $country) !== false && $store->getId() != $currentStoreId && $useMultistores
                        ) {
                            if ($logEnabled) {
                                $this->_logger->info("Geolocalized, redirect to " . $redirectStoreUrl);
                            }

                            $this->_helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            $controller->getResponse()->setRedirect($redirectStoreUrl);
                            $controller->getResponse()->sendResponse();
                            exit;
                        }
                        
                         if ($country && $countries && strpos($countries, $country) !== false && $store->getId() == $currentStoreId
                        ) {
                            if ($logEnabled) {
                                $this->_logger->info("Already in the right website, set only cookie no redirect necessary");
                            }

                            $this->_helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            break;
                        }
                    }
                }
            }

            if (!$getAmYetRedirectCurrency && $this->scopeConfig->getValue('amgeoipredirect/country_currency/enable_currency')) {
                $currencyMapping = unserialize($this->scopeConfig->getValue('amgeoipredirect/country_currency/currency_mapping', $scopeStores, $currentStoreId));
                foreach ($currencyMapping as $countries => $currency) {
                    if (strpos($countries, $country) !== false && $this->storeManager->getStore()->getCurrentCurrencyCode() != $currency) {
                        $this->_helper->setCurrencyCookie();
                        $this->storeManager->getStore()->setCurrentCurrencyCode($currency);
                    }
                }
            }
        }

        if ($logEnabled) {
            $this->_logger->info(" #### END REQUEST #### ");
        }
    }

}
